#include <openvr.h>
#include "openvrcombiner.h"
#include "openvrsystem.h"

namespace Msp {
namespace VR {

struct OpenVRCombiner::Private
{
	static Frustum get_projection(vr::EVREye);
};


OpenVRCombiner::OpenVRCombiner(OpenVRSystem &d, GL::View &v):
	system(d),
	view(v)
{
	vr::IVRSystem *vr_sys = vr::VRSystem();
	uint32_t w, h;
	vr_sys->GetRecommendedRenderTargetSize(&w, &h);
	target_width = w;
	target_height = h;

	Frustum left_frustum = Private::get_projection(vr::Eye_Left);
	Frustum right_frustum = Private::get_projection(vr::Eye_Right);
	configure_eye_frustums(left_frustum, right_frustum);

	view.get_context().set_swap_interval(0);
	set_mirroring(true);
}

void OpenVRCombiner::prepare() const
{
	system.update_pose_matrices();
}

void OpenVRCombiner::render(const GL::Texture2D &left, const GL::Texture2D &right) const
{
	vr::Texture_t tex;
	tex.eType = vr::API_OpenGL;
	tex.eColorSpace = vr::ColorSpace_Gamma;

	vr::IVRCompositor *compositor = vr::VRCompositor();
	tex.handle = reinterpret_cast<void *>(left.get_id());
	compositor->Submit(vr::Eye_Left, &tex);
	tex.handle = reinterpret_cast<void *>(right.get_id());
	compositor->Submit(vr::Eye_Right, &tex);

	if(mirror)
	{
		mirror->shdata.uniform("scale", view.get_aspect()/render_aspect/2, 0.5f);
		render_mirror(left);
		view.get_context().swap_buffers();
	}
}


StereoCombiner::Frustum OpenVRCombiner::Private::get_projection(vr::EVREye eye)
{
	Frustum frustum;
	/* The parameter order is documented as left, right, top, bottom; however
	the third parameter is actually negative and fourth one positive. */
	vr::VRSystem()->GetProjectionRaw(eye, &frustum.left, &frustum.right, &frustum.bottom, &frustum.top);
	return frustum;
}

} // namespace VR
} // namespace Msp
